@@ -1,7 +1,7 @@
 ##
 ##  OSSP uuid - Universally Unique Identifier
-##  Copyright (c) 2004-2005 Ralf S. Engelschall <rse@engelschall.com>
-##  Copyright (c) 2004-2005 The OSSP Project <http://www.ossp.org/>
+##  Copyright (c) 2004-2006 Ralf S. Engelschall <rse@engelschall.com>
+##  Copyright (c) 2004-2006 The OSSP Project <http://www.ossp.org/>
 ##
 ##  This file is part of OSSP uuid, a library for the generation
 ##  of UUIDs which can found at http://www.ossp.org/pkg/lib/uuid/
@@ -27,14 +27,14 @@
 ##  uuid.ts: Perl Binding (Perl test suite part)
 ##
 
-use Test::More tests => 31;
+use Test::More tests => 36;
 
 ##
 ##  Module Loading
 ##
 
 BEGIN {
-    use_ok('OSSP::uuid')
+    use_ok('OSSP::uuid');
 };
 BEGIN {
     use OSSP::uuid qw(:all);
@@ -87,6 +87,10 @@ $rc = uuid_export($uuid_ns, UUID_FMT_STR, $ptr, $len);
 ok((    $rc == UUID_RC_OK
     and $ptr eq "02d9e6d5-9467-382e-8f9b-9300a64ac3cd"
     and $len == UUID_LEN_STR), "uuid_export (5)");
+$rc = uuid_export($uuid_ns, UUID_FMT_SIV, $ptr, $len);
+ok((    $rc == UUID_RC_OK
+    and $ptr eq "3789866285607910888100818383505376205"
+    and $len <= UUID_LEN_SIV), "uuid_export (6)");
 
 $rc = uuid_destroy($uuid_ns);
 ok($rc == UUID_RC_OK, "uuid_destroy (1)");
@@ -142,3 +146,26 @@ ok((    defined($ptr)
 undef $uuid;
 undef $uuid_ns;
 
+##
+##  TIE API
+##
+
+$uuid = new OSSP::uuid;
+
+tie my $var, 'OSSP::uuid::tie';
+
+my $val_get1 = $var;
+my $val_get2 = $var;
+ok($val_get1 ne $val_get2, "subsequent generation");
+
+$uuid->import("str", $val_get1);
+my $val_cmp1 = $uuid->export("str");
+$uuid->import("str", $val_get2);
+my $val_cmp2 = $uuid->export("str");
+ok($val_get1 eq $val_cmp1, "validity comparison 1");
+ok($val_get2 eq $val_cmp2, "validity comparison 2");
+
+$var = [ "v3", "ns:URL", "http://www.ossp.org/" ];
+$val_get1 = $var;
+ok($val_get1 eq "02d9e6d5-9467-382e-8f9b-9300a64ac3cd", "generation of UUID v3");
+
